@@ -12,8 +12,9 @@ from collections.abc import (
     Mapping,
     MutableMapping,
 )
-from contextlib import ContextDecorator, contextmanager, suppress
+from contextlib import contextmanager, suppress
 from dataclasses import dataclass, field
+from enum import Enum
 from functools import partialmethod, singledispatchmethod, update_wrapper
 from inspect import Signature, isclass
 from types import MethodType, UnionType
@@ -26,8 +27,6 @@ from typing import (
     NoReturn,
     Protocol,
     TypeVar,
-    cast,
-    get_args,
     runtime_checkable,
 )
 
@@ -45,21 +44,12 @@ from injection.exceptions import (
     NoInjectable,
 )
 
-__all__ = ("Injectable", "Module")
+__all__ = ("Injectable", "Mode", "Module", "Priority")
 
 _logger = logging.getLogger(__name__)
 
 _T = TypeVar("_T")
-_Types = Iterable[type] | UnionType
-
-
-"""
-Literal types
-"""
-
-
-_Mode = Literal["fallback", "normal", "override"]
-_Priority = Literal["low", "high"]
+_T_co = TypeVar("_T_co", covariant=True)
 
 
 """
@@ -75,7 +65,7 @@ class ContainerEvent(Event, ABC):
 @dataclass(frozen=True, slots=True)
 class ContainerDependenciesUpdated(ContainerEvent):
     classes: Collection[type]
-    mode: _Mode
+    mode: Mode
 
     def __str__(self) -> str:
         length = len(self.classes)
@@ -113,6 +103,7 @@ class ModuleEventProxy(ModuleEvent):
 @dataclass(frozen=True, slots=True)
 class ModuleAdded(ModuleEvent):
     module_added: Module
+    priority: Priority
 
     def __str__(self) -> str:
         return f"`{self.on_module}` now uses `{self.module_added}`."
@@ -129,7 +120,7 @@ class ModuleRemoved(ModuleEvent):
 @dataclass(frozen=True, slots=True)
 class ModulePriorityUpdated(ModuleEvent):
     module_updated: Module
-    priority: _Priority
+    priority: Priority
 
     def __str__(self) -> str:
         return (
@@ -144,10 +135,10 @@ Injectables
 
 
 @runtime_checkable
-class Injectable(Protocol[_T]):
+class Injectable(Protocol[_T_co]):
     __slots__ = ()
 
-    def __init__(self, __factory: Callable[[], _T] = None, /):
+    def __init__(self, __factory: Callable[[], _T_co] = None, /):
         pass
 
     @property
@@ -155,10 +146,10 @@ class Injectable(Protocol[_T]):
         return False
 
     def unlock(self):
-        pass
+        return
 
     @abstractmethod
-    def get_instance(self) -> _T:
+    def get_instance(self) -> _T_co:
         raise NotImplementedError
 
 
@@ -241,20 +232,37 @@ Container
 """
 
 
+class Mode(str, Enum):
+    FALLBACK = "fallback"
+    NORMAL = "normal"
+    OVERRIDE = "override"
+
+    @property
+    def rank(self) -> int:
+        return tuple(type(self)).index(self)
+
+    @classmethod
+    def get_default(cls):
+        return cls.NORMAL
+
+
+ModeStr = Literal["fallback", "normal", "override"]
+
+
 class Record(NamedTuple):
     injectable: Injectable
-    mode: _Mode
+    mode: Mode
 
 
 @dataclass(repr=False, frozen=True, slots=True)
 class Container(Broker):
-    __data: dict[type, Record] = field(default_factory=dict, init=False)
+    __records: dict[type, Record] = field(default_factory=dict, init=False)
     __channel: EventChannel = field(default_factory=EventChannel, init=False)
 
     def __getitem__(self, cls: type[_T] | UnionType, /) -> Injectable[_T]:
         for cls in get_origins(cls):
             try:
-                injectable, _ = self.__data[cls]
+                injectable, _ = self.__records[cls]
             except KeyError:
                 continue
 
@@ -263,7 +271,7 @@ class Container(Broker):
         raise NoInjectable(cls)
 
     def __contains__(self, cls: type | UnionType, /) -> bool:
-        return any(cls in self.__data for cls in get_origins(cls))
+        return any(cls in self.__records for cls in get_origins(cls))
 
     @property
     def is_locked(self) -> bool:
@@ -271,20 +279,26 @@ class Container(Broker):
 
     @property
     def __injectables(self) -> frozenset[Injectable]:
-        return frozenset(injectable for injectable, _ in self.__data.values())
+        return frozenset(injectable for injectable, _ in self.__records.values())
 
     @synchronized()
-    def update(self, classes: Iterable[type], injectable: Injectable, mode: _Mode):
+    def update(
+        self,
+        classes: Iterable[type | UnionType],
+        injectable: Injectable,
+        mode: Mode | ModeStr,
+    ):
+        mode = Mode(mode)
         records = {
             cls: Record(injectable, mode)
-            for cls in self.__filter_classes(classes, mode)
+            for cls in self.__classes_to_update(classes, mode)
         }
 
         if records:
             event = ContainerDependenciesUpdated(self, records.keys(), mode)
 
             with self.notify(event):
-                self.__data.update(records)
+                self.__records.update(records)
 
         return self
 
@@ -299,16 +313,19 @@ class Container(Broker):
         self.__channel.add_listener(listener)
         return self
 
-    def notify(self, event: Event) -> ContextManager | ContextDecorator:
+    def notify(self, event: Event):
         return self.__channel.dispatch(event)
 
-    def __filter_classes(self, classes: Iterable[type], mode: _Mode) -> Iterator[type]:
-        modes = get_args(_Mode)
-        rank = modes.index(mode)
+    def __classes_to_update(
+        self,
+        classes: Iterable[type | UnionType],
+        mode: Mode,
+    ) -> Iterator[type]:
+        rank = mode.rank
 
         for cls in frozenset(get_origins(*classes)):
             try:
-                _, current_mode = self.__data[cls]
+                _, current_mode = self.__records[cls]
 
             except KeyError:
                 pass
@@ -319,7 +336,7 @@ class Container(Broker):
                         f"An injectable already exists for the class `{format_type(cls)}`."
                     )
 
-                elif rank < modes.index(current_mode):
+                elif rank < current_mode.rank:
                     continue
 
             yield cls
@@ -330,9 +347,21 @@ Module
 """
 
 
+class Priority(str, Enum):
+    LOW = "low"
+    HIGH = "high"
+
+    @classmethod
+    def get_default(cls):
+        return cls.LOW
+
+
+PriorityStr = Literal["low", "high"]
+
+
 @dataclass(repr=False, eq=False, frozen=True, slots=True)
 class Module(EventListener, Broker):
-    name: str = field(default=None)
+    name: str | None = field(default=None)
     __channel: EventChannel = field(default_factory=EventChannel, init=False)
     __container: Container = field(default_factory=Container, init=False)
     __modules: OrderedDict[Module, None] = field(
@@ -375,8 +404,8 @@ class Module(EventListener, Broker):
         *,
         cls: type[Injectable] = NewInjectable,
         inject: bool = True,
-        on: type | _Types = None,
-        mode: _Mode = "normal",
+        on: type | Iterable[type] | UnionType = (),
+        mode: Mode | ModeStr = Mode.get_default(),
     ):
         def decorator(wp):
             factory = self.inject(wp, return_factory=True) if inject else wp
@@ -394,7 +423,7 @@ class Module(EventListener, Broker):
             self.update(
                 (wp,),
                 ShouldBeInjectable(wp),
-                mode="fallback",
+                mode=Mode.FALLBACK,
             )
             return wp
 
@@ -403,15 +432,15 @@ class Module(EventListener, Broker):
     def set_constant(
         self,
         instance: _T,
-        on: type | _Types = None,
+        on: type | Iterable[type] | UnionType = (),
         *,
-        mode: _Mode = "normal",
+        mode: Mode | ModeStr = Mode.get_default(),
     ) -> _T:
         cls = type(instance)
         self.injectable(
             lambda: instance,
             inject=False,
-            on=(cls, on),
+            on=(cls, on),  # type: ignore
             mode=mode,
         )
         return instance
@@ -439,17 +468,15 @@ class Module(EventListener, Broker):
 
         return decorator(wrapped) if wrapped else decorator
 
-    def get_instance(self, cls: type[_T], *, none: bool = True) -> _T | None:
+    def resolve(self, cls: type[_T]) -> _T:
+        injectable = self[cls]
+        return injectable.get_instance()
+
+    def get_instance(self, cls: type[_T]) -> _T | None:
         try:
-            injectable = self[cls]
-        except KeyError as exc:
-            if none:
-                return None
-
-            raise exc
-
-        instance = injectable.get_instance()
-        return cast(cls, instance)
+            return self.resolve(cls)
+        except KeyError:
+            return None
 
     def get_lazy_instance(
         self,
@@ -466,21 +493,27 @@ class Module(EventListener, Broker):
 
     def update(
         self,
-        classes: Iterable[type],
+        classes: Iterable[type | UnionType],
         injectable: Injectable,
-        mode: _Mode = "normal",
+        mode: Mode | ModeStr = Mode.get_default(),
     ):
         self.__container.update(classes, injectable, mode)
         return self
 
-    def use(self, module: Module, *, priority: _Priority = "low"):
+    def use(
+        self,
+        module: Module,
+        *,
+        priority: Priority | PriorityStr = Priority.get_default(),
+    ):
         if module is self:
             raise ModuleError("Module can't be used by itself.")
 
         if module in self.__modules:
             raise ModuleError(f"`{self}` already uses `{module}`.")
 
-        event = ModuleAdded(self, module)
+        priority = Priority(priority)
+        event = ModuleAdded(self, module, priority)
 
         with self.notify(event):
             self.__modules[module] = None
@@ -504,13 +537,14 @@ class Module(EventListener, Broker):
         self,
         module: Module,
         *,
-        priority: _Priority = "low",
-    ) -> ContextManager | ContextDecorator:
+        priority: Priority | PriorityStr = Priority.get_default(),
+    ):
         self.use(module, priority=priority)
         yield
         self.stop_using(module)
 
-    def change_priority(self, module: Module, priority: _Priority):
+    def change_priority(self, module: Module, priority: Priority | PriorityStr):
+        priority = Priority(priority)
         event = ModulePriorityUpdated(self, module, priority)
 
         with self.notify(event):
@@ -538,7 +572,7 @@ class Module(EventListener, Broker):
         return self.notify(self_event)
 
     @contextmanager
-    def notify(self, event: Event) -> ContextManager | ContextDecorator:
+    def notify(self, event: Event):
         self.__check_locking()
 
         with self.__channel.dispatch(event):
@@ -549,8 +583,8 @@ class Module(EventListener, Broker):
         if self.is_locked:
             raise ModuleLockError(f"`{self}` is locked.")
 
-    def __move_module(self, module: Module, priority: _Priority):
-        last = priority != "high"
+    def __move_module(self, module: Module, priority: Priority):
+        last = priority != Priority.HIGH
 
         try:
             self.__modules.move_to_end(module, last=last)
@@ -609,7 +643,7 @@ class Dependencies:
     ) -> Iterator[tuple[str, Injectable]]:
         for name, annotation in cls.__get_annotations(signature, owner):
             try:
-                injectable = module[annotation]
+                injectable: Injectable = module[annotation]
             except KeyError:
                 continue
 
@@ -646,32 +680,25 @@ class InjectedFunction(EventListener):
     )
 
     def __init__(self, wrapped: Callable[..., Any], /):
-        try:
-            variables = vars(wrapped)
-        except TypeError:
-            pass
-        else:
-            self.__update_vars(variables)
-            del variables
-
+        self.__update_vars_from(wrapped)
         update_wrapper(self, wrapped, updated=())
         self.__dependencies = Dependencies.empty()
         self.__owner = None
         self.__setup_queue = LimitedQueue[Callable[[], Any]]()
         self.on_setup(self.__set_signature)
 
-    def __repr__(self) -> str:
+    def __repr__(self) -> str:  # pragma: no cover
         return repr(self.wrapped)
 
-    def __str__(self) -> str:
+    def __str__(self) -> str:  # pragma: no cover
         return str(self.wrapped)
 
     def __call__(self, /, *args, **kwargs) -> Any:
         for function in self.__setup_queue:
             function()
 
-        args, kwargs = self.bind(args, kwargs)
-        return self.wrapped(*args, **kwargs)
+        arguments = self.bind(args, kwargs)
+        return self.wrapped(*arguments.args, **arguments.kwargs)
 
     def __get__(self, instance: object = None, owner: type = None):
         if instance is None:
@@ -688,7 +715,7 @@ class InjectedFunction(EventListener):
 
     @property
     def wrapped(self) -> Callable[..., Any]:
-        return self.__wrapped__
+        return self.__wrapped__  # type: ignore
 
     def bind(
         self,
@@ -716,7 +743,7 @@ class InjectedFunction(EventListener):
         if self.__owner:
             raise TypeError("Function owner is already defined.")
 
-        self.__owner = owner
+        self.__owner = owner  # type: ignore
         return self
 
     @synchronized()
@@ -732,12 +759,12 @@ class InjectedFunction(EventListener):
         return decorator(wrapped) if wrapped else decorator
 
     @singledispatchmethod
-    def on_event(self, event: Event, /):
-        pass
+    def on_event(self, event: Event, /) -> ContextManager | None:  # type: ignore
+        return None
 
     @on_event.register
     @contextmanager
-    def _(self, event: ModuleEvent, /) -> ContextManager:
+    def _(self, event: ModuleEvent, /):
         yield
         self.update(event.on_module)
 
@@ -745,14 +772,21 @@ class InjectedFunction(EventListener):
         self.__signature__ = inspect.signature(self.wrapped, eval_str=True)
         return self
 
-    def __update_vars(self, variables: Mapping[str, Any], /):
+    def __update_vars_from(self, obj: Any):
+        try:
+            variables = vars(obj)
+        except TypeError:
+            pass
+        else:
+            self.__update_vars(variables)
+
+    def __update_vars(self, variables: Mapping[str, Any]):
         def is_dunder(var: str) -> bool:
             return var.startswith("__") and var.endswith("__")
 
         restricted_vars = frozenset(var for var in dir(self) if not is_dunder(var))
-        variables = (
+        vars(self).update(
             (var, value)
             for var, value in variables.items()
             if var not in restricted_vars
         )
-        vars(self).update(variables)
